# coding: utf-8
import requests
from django.test import mock
from django.contrib.auth.models import User
from django.conf import settings
from rest_framework import test, status
from rest_framework.authtoken.models import Token
from .. import authentication


class AuthenticationTestCase(test.APITestCase):

    def setUp(self):
        user = User.objects.create(username='tester')
        self.token = Token.objects.create(user=user)
        self.header = {'HTTP_AUTHORIZATION': 'TokenService ' + self.token.key}
        self.request_factory = test.APIRequestFactory()
        self.request = self.request_factory.get('/some_url/', **self.header)

    @mock.patch('sw_rest_auth.authentication.requests')
    def test_success_request(self, requests_mock):
        requests_mock.post.return_value.status_code = status.HTTP_200_OK
        requests_mock.post.return_value.json.return_value = {'username': 'test_username'}

        user, _ = authentication.TokenServiceAuthentication().authenticate(self.request)

        call_args, call_kwargs = requests_mock.post.call_args
        self.assertEqual(settings.AUTH_SERVICE_CHECK_TOKEN_URL, call_args[0])
        self.assertEqual(self.token.key, call_kwargs['data']['token'])
        self.assertEqual(user.username, requests_mock.post.return_value.json.return_value['username'])

    @mock.patch('sw_rest_auth.authentication.requests')
    def test_validation_error(self, requests_mock):
        requests_mock.post.return_value.status_code = status.HTTP_400_BAD_REQUEST
        requests_mock.post.return_value.json.return_value = {'token': 'token error'}

        auth_ex = None
        try:
            authentication.TokenServiceAuthentication().authenticate(self.request)
        except authentication.exceptions.AuthenticationFailed as ex:
            auth_ex = ex

        self.assertTrue(requests_mock.post.called)
        self.assertIsNotNone(auth_ex)

    @mock.patch('sw_rest_auth.authentication.requests.head', side_effect=requests.ConnectionError)
    def test_connection_error(self, requests_mock):
        auth_ex = None
        try:
            authentication.TokenServiceAuthentication().authenticate(self.request)
        except authentication.exceptions.AuthenticationFailed as ex:
            auth_ex = ex
        self.assertIsNotNone(auth_ex)




# coding: utf-8
from django.contrib.auth.models import User
from rest_framework import serializers
from rest_framework.authtoken.models import Token
from . import models


class CheckToken(serializers.Serializer):
    token = serializers.CharField()

    def validate_token(self, value):
        try:
            return Token.objects.get(key=value)
        except Token.DoesNotExist:
            raise serializers.ValidationError('Token not found')


class CheckPerm(serializers.Serializer):
    user = serializers.CharField()
    perm = serializers.CharField(max_length=50)

    def validate_user(self, value):
        try:
            return User.objects.get(username=value)
        except User.DoesNotExist:
            raise serializers.ValidationError('User with username "{0}" not found'.format(value))

    def validate_perm(self, value):
        try:
            return models.Permission.objects.get(code=value)
        except models.Permission.DoesNotExist:
            raise serializers.ValidationError('Permission with code "{0}" not found'.format(value))

    def validate(self, attrs):
        user = attrs.get('user')
        perm = attrs.get('perm')
        if user and perm:
            try:
                models.UserPermission.objects.get(user=user, permission=perm)
            except models.UserPermission.DoesNotExist:
                raise serializers.ValidationError('User "{0}" have no permission with code "{1}"'.format(
                    user.username,
                    perm.code
                ))

        return attrs



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-18 00:01
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Permission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(help_text='code for using in permission class in program code', max_length=50, unique=True)),
                ('name', models.CharField(help_text='human permission name', max_length=100, unique=True)),
                ('description', models.TextField(blank=True, help_text='detailed description')),
            ],
        ),
        migrations.CreateModel(
            name='UserPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('permission', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sw_rest_auth.Permission')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='userpermission',
            unique_together=set([('user', 'permission')]),
        ),
    ]



# coding: utf-8
from django.conf import settings
from rest_framework import permissions
import requests
from rest_framework.exceptions import PermissionDenied


class CodePermission(permissions.BasePermission):
    def __init__(self, permission_code):
        super(CodePermission, self).__init__()
        self.permission_code = permission_code

    def has_permission(self, request, view):
        if not (request.user and request.user.username):
            return False

        url = settings.AUTH_SERVICE_CHECK_PERM_URL
        auth_token = settings.AUTH_TOKEN

        headers = {'Authorization': 'Token %s' % auth_token}
        params = {'user': request.user.username, 'perm': self.permission_code}
        try:
            r = requests.get(url, headers=headers, params=params)
        except requests.ConnectionError:
            raise PermissionDenied(detail='Can not connect to authorization service')

        if r.status_code != 200:
            detail = '; '.join(['%s: %s' % (k, ', '.join(v)) for k, v in r.json().items()])
            raise PermissionDenied(detail=detail)

        return True

    @classmethod
    def decorate(cls, code):
        def decorator():
            return cls(permission_code=code)
        return decorator



# coding: utf-8
from django.db import models
from django.contrib.auth.models import User


class Permission(models.Model):
    code = models.CharField(max_length=50, unique=True, help_text='code for using in permission class in program code')
    name = models.CharField(max_length=100, unique=True, help_text='human permission name')
    description = models.TextField(blank=True, help_text='detailed description')

    def __str__(self):
        return '{0} ({1})'.format(self.name, self.code)


class UserPermission(models.Model):
    user = models.ForeignKey(User)
    permission = models.ForeignKey(Permission)

    class Meta:
        unique_together = ('user', 'permission')

    def __str__(self):
        return '{0} - {1}'.format(self.user, self.permission)



# coding: utf-8
from django.core.urlresolvers import reverse
from django.test import mock
from django.contrib.auth.models import User
from django.conf import settings
from rest_framework import test, status
from rest_framework.authtoken.models import Token
from .. import models
from .. import views
from .helpers import AuthTestCaseMixin


class CheckToken(AuthTestCaseMixin, test.APITestCase):
    url = reverse(views.check_token)

    def test_success(self):
        token = Token.objects.create(user=self.get_user())
        response = self.client.post(self.url, {'token': token.key})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['username'], token.user.username)

    def test_invalid_key(self):
        response = self.client.post(self.url, {'token': 'not_existing_token_key'})
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class CheckPerm(AuthTestCaseMixin, test.APITestCase):
    url = reverse(views.check_perm)

    def setUp(self):
        super(CheckPerm, self).setUp()
        self.permission = models.Permission.objects.create(code='code', name='name', description='description')

    def test_have_perm(self):
        models.UserPermission.objects.create(permission=self.permission, user=self.get_user())
        response = self.client.get(self.url, {'user': self.get_user().username, 'perm': self.permission.code})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_have_no_perm(self):
        models.UserPermission.objects.filter(permission=self.permission, user=self.get_user()).delete()
        response = self.client.get(self.url, {'user': self.get_user().username, 'perm': self.permission.code})
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)



# coding: utf-8
from django.contrib.auth.models import User
from django.conf import settings
from rest_framework import exceptions
from rest_framework.authentication import BaseAuthentication, get_authorization_header
import requests


class TokenServiceAuthentication(BaseAuthentication):
    """
    Token based authentication by means third part services.

    Clients should authenticate by passing the token key in the "Authorization"
    HTTP header, prepended with the string "TokenService ".  For example:

        Authorization: TokenService 401f7ac837da42b97f613d789819ff93537bee6a
    """

    def authenticate(self, request):
        auth = get_authorization_header(request).split()

        if not auth or auth[0].lower() != b'tokenservice':
            return None

        if len(auth) == 1:
            msg = 'Invalid token header. No credentials provided.'
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth) > 2:
            msg = 'Invalid token header. Token string should not contain spaces.'
            raise exceptions.AuthenticationFailed(msg)

        try:
            token_key = auth[1].decode()
        except UnicodeError:
            msg = 'Invalid token header. Token string should not contain invalid characters.'
            raise exceptions.AuthenticationFailed(msg)

        return self._check_token(token_key)

    @staticmethod
    def _check_token(token_key):
        url = settings.AUTH_SERVICE_CHECK_TOKEN_URL
        auth_token = settings.AUTH_TOKEN

        headers = {'Authorization': 'Token %s' % auth_token}
        data = {'token': token_key}
        try:
            r = requests.post(url, headers=headers, data=data)
        except requests.ConnectionError:
            raise exceptions.AuthenticationFailed('Invalid token header. ConnectionError.')

        if r.status_code == 200:
            result = r.json()
            user = User(username=result['username'])
            return user, None

        elif r.status_code == 400:
            result = r.json()
            token_err_description = ', '.join(result['token'])
            raise exceptions.AuthenticationFailed('Invalid token header. %s' % token_err_description)

        else:
            raise exceptions.AuthenticationFailed('Invalid token header. Unknown error: %s' % r.text)

    def authenticate_header(self, request):
        return 'TokenService'
